#!/usr/bin/python

"""
Author: Jan Havlik (xhavli43@stud.fit.vutbr.cz) 1MSK

Script for a mobile communication analysis from a pcap files

Working environment:
  - Linux 4.9.0-kali3-amd64 x86_64
  - Server merlin.fit.vutbr.cz
  - Server eva.fit.vutbr.cz (FreeBSD 10.4-STABLE amd64)

  !!! Python v3 !!!

Init:
  python3 -f <path_to_file>
"""

import sys
import os
sys.path.append("libraries/scapy-master")
import logging
logging.getLogger("scapy.runtime").setLevel(logging.ERROR)
from scapy.all import *
import json
import os.path
import re


"""
Custom files
"""
from obj import *

"""
GLOBAL VARIABLES
"""
pktTimeout = None # set to infinity if None
pktCount = 0 # set to infinity if None
live = False
"""
Process the cmd arguments
"""
argcnt = len(sys.argv)
arglist = sys.argv
filename = ""

if (argcnt < 2) or (argcnt > 3):
    print("Incorrect number of arguments.\nPlease use -h for help.")
    exit(1)
else:
    if (argcnt == 2):
        if (arglist[1] == "-h"):
            print("Help.")
            print( "Use 'python mal.py -f <filename>' to start analysing a file.")
            print( "Use 'python mal.py -l' to start a mobile sniffer. You HAVE to be the root for this action.")
            exit(0)
        elif (arglist[1] == "-l"):
            live = True
        else:
            print( "Incorrect parameter usage.\nPlease use -h for help.")
            exit(1)
    else: # argcnt==3
        if (arglist[1] == "-f"):
            filename = arglist[2]
        else:
            print( "Incorrect parameter usage.\nPlease use -h for help.")
            exit(1)

"""
Check if filename exists
"""
if not live:
    if not (os.path.exists(filename)):
        print( "File does not exist! Terminating...")
        exit(1)

    if not (os.path.exists("config.json")):
        print( "No config file!\nConfig file has to be in the same folder as 'mal.py'")
        exit(1)

"""
Loading the configuration file
"""
try:
    with open('config.json') as data_file:
        config = json.load(data_file)
except EnvironmentError: # parent of IOError, OSError *and* WindowsError where available
    print('\nError: Config file not found\nThe config.json file must be in this directory.')
    exit(1)

"""
USER FUNCTIONS
"""

def findUpdateMac(devices, cmpMac, addNew, strIp, strOs, strVersion, strUaInfo, strdevType, strHostname, strLang, cnt, config):
    # devices is an array of all devices
    # cmpMac is a max str for comparing
    # update if true, updates found device
    # returns True if found

    devFound = False

    for device in devices:
        if (device.mac == cmpMac):
            device.update(strIp, strOs, strVersion, strUaInfo, strdevType, strHostname, strLang, cnt)
            devFound = True # found a device

    if addNew == True:
        if not devFound:
            dev = Device(strIp, cmpMac, strOs, strVersion, strUaInfo, strdevType, strHostname, strLang) # generic
            dev.loadPorts(config["customPortNum"])
            devices.append(dev)

    return devFound


def httpUpdate(devices, httpFieldName, httpFieldValue, cmpMac):
    devFound = False
    for device in devices:
        if (device.mac == cmpMac):
            device.httpUpdate(httpFieldName, httpFieldValue)
            devFound = True # found a device

"""
********************
*** MAIN PROGRAM ***
********************
"""
cnt = 0
cntSec = 0
ouiFilters = []
dnsFilters = []
dhcpFilters = []
devices = []
UserAgentStrings = []

ouiFilters = ouiFilter(config["oui"])
ouiFilters = ouiFilters.parseFilter()

dnsFilters = dnsFilter(config["dns"])
dnsFilters = dnsFilters.parseFilter()

UserAgentStrings = uaFilter(config["userAgent"])
UserAgentStrings = UserAgentStrings.parseStrings()

dhcpFilters = dhcpFilter(config["dhcpHostName"])
dhcpFilters = dhcpFilters.parseStrings()

#portFilters = portFilter(config["customPortNum"])
#portFilters = portFilters.parsePorts()

dnsResolve = {}

outName = filename.split('/')[-1]
outName = outName.split('.')[0]
f = open(outName + '.csv', 'w')
fTxt = open(outName + '.txt', 'w')


def analyseLive(pkt):
    global cnt, cntSec
    cnt += 1

    strIp = "?"
    strOs = "?"
    strVersion = "?"
    strUaInfo = "?"
    strdevType = "?"
    strHostname = "?"
    strMac = "?"
    strLang = "?"

    headersVar = ""

    # MAC addr translator
    if (Ether in pkt) or (CookedLinux in pkt):
        if (CookedLinux in pkt):
            mac = pkt[CookedLinux].src.hex()
            oui = mac[0:2]+':'+mac[2:4]+':'+mac[4:6]
            strMac = mac[0:2]+':'+mac[2:4]+':'+mac[4:6]+":"+mac[6:8]+':'+mac[8:10]+':'+mac[10:12]
        elif (Ether in pkt):
            mac = pkt[Ether].src.split(':')
            oui = mac[0]+':'+mac[1]+':'+mac[2]
            strMac = mac[0]+':'+mac[1]+':'+mac[2]+":"+mac[3]+':'+mac[4]+':'+mac[5]

            macDst = pkt[Ether].dst.split(':')
            strMacDst = macDst[0]+':'+macDst[1]+':'+macDst[2]+":"+macDst[3]+':'+macDst[4]+':'+macDst[5]

        if TCP in pkt:
            if (pkt[TCP].dport == 443) or (pkt[TCP].sport == 443): # SSL/QUIC/HTTPS
                cntSec += 1
        elif UDP in pkt:
            if (pkt[UDP].dport == 443) or (pkt[UDP].sport == 443):# SSL/QUIC/HTTPS
                cntSec += 1

        # DHCP
        if DHCP in pkt:
            for opt in pkt[DHCP].options:
                if opt[0]=="hostname":
                    strHostname = opt[1].decode('UTF-8')
                    devFound = False
                    for flt in dhcpFilters:
                        if (strHostname.find(flt) != -1):
                            for device in devices:
                                if (device.mac == strMac):
                                    devFound = True
                            if not devFound:
                                dev = Device(strIp, strMac, strOs, strVersion, strUaInfo, strdevType, strHostname, strLang) # generic
                                dev.loadPorts(config["customPortNum"])
                                devices.append(dev)

        # OUI
        if (IP in pkt):
            for flt in ouiFilters:
                if flt["oui"] == oui:
                    strIp = pkt[IP].src
                    strOs = flt["os"]
                    strVersion = flt["version"]
                    strdevType = flt["type"]

                    addNew = True
                    devFound = findUpdateMac(devices, strMac, addNew, strIp, strOs, strVersion, strUaInfo, strdevType, strHostname, strLang, cnt, config)

        # DNS filters
        if (DNSRR in pkt):
            if (isinstance(pkt[DNSRR].rdata, str)):
                dnsResolve[pkt[DNSRR].rrname.decode('UTF-8')] = pkt[DNSRR].rdata
        if (DNSQR in pkt) and (UDP in pkt):
            if pkt[UDP].dport == 53:
                curQry = pkt[DNSQR].qname.decode('UTF-8')
                for flt in dnsFilters:
                    if curQry == flt["qry"]:
                        strIp = pkt[IP].src
                        strOs = flt["os"]
                        strVersion = flt["version"]

                        addNew = True
                        devFound = findUpdateMac(devices, strMac, addNew, strIp, strOs, strVersion, strUaInfo, strdevType, strHostname, strLang, cnt, config)


        # HTTP
        # TCP and only GET req so port is dst==80
        if TCP in pkt:
            if (pkt[TCP].dport==80) and Raw in pkt:
                rawStr = str(pkt[Raw])

                uaPos = rawStr.find("User-Agent")
                uaStr = ""
                langPos = rawStr.find("Accept-Language")
                langStr = ""
                cookPos = rawStr.find("Cookie:")

                # HTTP header parser
                # Change to \\r\\n when on EVA or MERLIN
                if (sys.platform == "linux2"):
                    headersVar = rawStr.split('\r\n')
                elif (sys.platform == "linux" or sys.platform == "freebsd10"):
                    headersVar = rawStr.split('\\r\\n')

                # parse http values into dict
                headersDict = {}
                for hdr in headersVar:
                    keyValHdr = hdr.split(':')
                    if (len(keyValHdr) > 1):
                        headersDict[str(keyValHdr[0])] = str(keyValHdr[1])

                # User-Agent
                if "User-Agent" in headersDict:
                    uaStr = headersDict["User-Agent"]

                    mobile = False
                    for uastr in UserAgentStrings:
                        try:
                            if (uaStr.find(uastr) != -1):
                                mobile = True
                        except UnicodeDecodeError:
                            mobile = False

                    if (mobile):
                        strIp = pkt[IP].src
                        strUaInfo = uaStr

                        addNew = True
                        devFound = findUpdateMac(devices, strMac, addNew, strIp, strOs, strVersion, strUaInfo, strdevType, strHostname, strLang, cnt, config)


                #Accept-Language
                if "Accept-Language" in headersDict:
                    langStr = headersDict["Accept-Language"]

                    strIp = pkt[IP].src
                    strLang = langStr

                    addNew = False # dont add new
                    devFound = findUpdateMac(devices, strMac, addNew, strIp, strOs, strVersion, strUaInfo, strdevType, strHostname, strLang, cnt, config)

                #Host
                if "Host" in headersDict:
                    devFound = httpUpdate(devices, "Host", headersDict["Host"], strMac)

                #Origin
                if "Origin" in headersDict:
                    devFound = httpUpdate(devices, "Origin", headersDict["Origin"], strMac)

                #Referer
                if "Referer" in headersDict:
                    devFound = httpUpdate(devices, "Referer", headersDict["Referer"], strMac)

                #Via
                if "Via" in headersDict:
                    devFound = httpUpdate(devices, "Via", headersDict["Via"], strMac)



            elif (pkt[TCP].sport==80) and Raw in pkt:
                rawStr = str(pkt[Raw])
                # HTTP header parser
                # Change to \\r\\n when on EVA or MERLIN
                if (sys.platform == "linux2"):
                    headersVar = rawStr.split('\r\n')
                elif (sys.platform == "linux"):
                    headersVar = rawStr.split('\\r\\n')


                # parse http values into dict
                headersDict = {}
                for hdr in headersVar:
                    keyValHdr = hdr.split(':')
                    if (len(keyValHdr) > 1):
                        headersDict[str(keyValHdr[0])] = str(keyValHdr[1])


                #Server
                if "Server" in headersDict:
                    devFound = httpUpdate(devices, "Server", headersDict["Server"], strMacDst)

                #Location
                if "Location" in headersDict:
                    devFound = httpUpdate(devices, "Location", headersDict["Location"], strMacDst)

                #Via
                if "Via" in headersDict:
                    devFound = httpUpdate(devices, "Via", headersDict["Via"], strMacDst)

        #### UPDATE STATS ####
        if IP in pkt:
            for device in devices:
                if (pkt[IP].src == device.ip):
                    if (TCP in pkt): #TCP
                        device.addDst(pkt[IP].dst, pkt[IP].len, "TCP", pkt[TCP].dport)
                    elif (UDP in pkt): #UDP
                        device.addDst(pkt[IP].dst, pkt[IP].len, "UDP", pkt[UDP].dport)
                elif (pkt[IP].dst == device.ip):
                    if (TCP in pkt): #TCP
                        device.addSrc(pkt[IP].src, pkt[IP].len, "TCP", pkt[TCP].sport)
                    elif (UDP in pkt): #UDP
                        device.addSrc(pkt[IP].src, pkt[IP].len, "UDP", pkt[UDP].sport)



# offline version
if live:
    sniff(prn=analyseLive, store=0, timeout=pktTimeout)
else:
    with PcapReader(filename) as pr:
      for pkt in pr:
        analyseLive(pkt)
    #pkts = rdpcap(filename)
    #for pkt in pkts:
    #    analyseLive(pkt)

prntCnt = 0
for dev in devices:
    dev.printInfo(f, fTxt, prntCnt, dnsResolve)
    prntCnt += 1

print("\nCiphered/Normal communication ratio: %.4g%%" % ((cntSec*100.0/cnt)))

f.close()
fTxt.close()
