#!/usr/bin/python

class Device:

    def __init__(self, ip, mac, os, version, uaInfo, devType, hostname, language):
        self.ip = ip
        self.mac = mac
        self.os = os
        self.version = version
        self.uaInfo = uaInfo
        self.type = devType
        self.hostname = hostname
        self.language = language
        self.stats = {"IpSrc": [], "IpDst": []}
        self.ipProtDict = {}

        #http headers
        self.httpHost = "?"
        self.httpOrigin = "?"
        self.httpReferer = "?"
        self.httpVia = "?"
        self.httpLocation = "?"
        self.httpServer = "?"

    # loads port information from json config file
    def loadPorts(self, portJson):
        self.ipProtDict = portJson[0]
        for portNum in self.ipProtDict:
            self.ipProtDict[portNum] = int(self.ipProtDict[portNum])

    #Outputs all the info
    def printInfo(self, fp, fpTxt, prntCnt, dnsResolve):

        outStr = "\nNew mobile device detected!"
        csvStr = ""

        if (prntCnt > 0):
            csvStr = " \r\n \r\n"
        csvStr += "IP address,MAC address,Operating system,Possible versions,Device type,User-Agent,Accept-Language,Host name, HTTP Hosts, HTTP Origin,HTTP Referer,HTTP Via,HTTP Location, HTTP Server \r\n"


        csvStr += self.ip+","
        csvStr += self.mac+","
        outStr += "\n********************"
        if (self.os != "?"):
            outStr += "\nOperating system:\n%s\n"  % (self.os)
            csvStr += self.os+","
        else:
            csvStr += " ,"

        if (self.version != "?"):
            outStr += "\nPossible versions:\n%s\n"  % (self.version)
            csvStr += self.version+","
        else:
            csvStr += " ,"
        if (self.type != "?"):
            outStr += "\nType of device:\n%s\n"  % (self.type)
            csvStr += self.type+","
        else:
            csvStr += " ,"

        outStr += "\nDevice addressing:\n%s (%s)\n"  % (self.ip, self.mac)

        if (self.uaInfo != "?"):
            outStr += "\nUser-Agent:\n%s\n" % (self.uaInfo)
            csvStr += "\""+(self.uaInfo)+"\""+","
        else:
            csvStr += " ,"

        if (self.language != "?"):
            try:
                outStr += "\nAccept-Language:\n%s\n" % (self.language)
                csvStr += "\""+self.language+"\""+","
            except UnicodeDecodeError:
                print()
        else:
            csvStr += " ,"

        if (self.hostname != "?"):
            outStr += "\nDHCP Host Name:\n%s\n" % (self.hostname)
            csvStr += "\""+self.hostname+"\""+","
        else:
            csvStr += " ,"

        if (self.httpHost != "?"):
            outStr += "\nHTTP Hosts:\n%s\n" % (self.httpHost)
            csvStr += "\""+self.httpHost+"\""+","
        else:
            csvStr += " ,"

        if (self.httpOrigin != "?"):
            outStr += "\nHTTP Origin:\n%s\n" % (self.httpOrigin)
            csvStr += "\""+self.httpOrigin+"\""+","
        else:
            csvStr += " ,"

        if (self.httpReferer != "?"):
            outStr += "\nHTTP Referer:\n%s\n" % (self.httpReferer)
            csvStr += "\""+self.httpReferer+"\""+","
        else:
            csvStr += " ,"

        if (self.httpVia != "?"):
            outStr += "\nHTTP Via:\n%s\n" % (self.httpVia)
            csvStr += "\""+self.httpVia+"\""+","
        else:
            csvStr += " ,"

        if (self.httpLocation != "?"):
            outStr += "\nHTTP Location:\n%s\n" % (self.httpLocation)
            csvStr += "\""+self.httpLocation+"\""+","
        else:
            csvStr += " ,"

        if (self.httpServer != "?"):
            outStr += "\nHTTP Server:\n%s\n" % (self.httpServer)
            csvStr += "\""+self.httpServer+"\""+","
        else:
            csvStr += " ,"

        csvStr += " \r\n"
        outStr += "\n===================="
        if not (self.ip == "?" or self.mac == "?"):
            print(outStr)

            if (fp != None):
                fp.write(csvStr)

            csvStr = " \r\n"
            csvStr += "Ip address -> Device, Total length [bytes], Protocols\r\n"
            print("\n*** DEVICE COMMUNICATION (Endpoint to Device) ***")
            for stat in self.stats["IpSrc"]:
                try:
                    print("%s [%s] -> %s" % (stat["Address"], list(dnsResolve.keys())[list(dnsResolve.values()).index(stat["Address"])], self.ip))
                except ValueError:
                    print(stat["Address"] + " -> " + self.ip)
                csvStr += stat["Address"] + ","+ str(stat["Volume"]) + ","
                for oneProt in stat["Statistics"]:
                    csvStr += str(oneProt["Protocol"]) + ","
                    try:
                        print("\tProtocol: %s (%d), Data: %d bytes" % (list(self.ipProtDict.keys())[list(self.ipProtDict.values()).index(oneProt["Protocol"])], oneProt["Protocol"], oneProt["Data"]))
                    except ValueError:
                        print("\tProtocol: %s (%d), Data: %d bytes" % ("Not in list", oneProt["Protocol"], oneProt["Data"]))
                csvStr += " \r\n"

            csvStr += " \r\n"
            csvStr += "Device -> IP address, Total length [bytes], Protocols\r\n"
            print("\n*** DEVICE COMMUNICATION (Device to Endpoint) ***")
            for stat in self.stats["IpDst"]:
                try:
                    outStr += "%s -> %s [%s]\n" % (self.ip, stat["Address"], list(dnsResolve.keys())[list(dnsResolve.values()).index(stat["Address"])])
                    print("%s -> %s [%s]" % (self.ip, stat["Address"], list(dnsResolve.keys())[list(dnsResolve.values()).index(stat["Address"])]))
                except ValueError:
                    outStr += "\n"
                    outStr += self.ip + " -> " + stat["Address"] + " (%d bytes)\n" % (stat["Volume"])
                    print(self.ip + " -> " + stat["Address"] + " (%d bytes)" % (stat["Volume"]))
                csvStr += stat["Address"] + ","+ str(stat["Volume"]) + ","
                for oneProt in stat["Statistics"]:
                    csvStr += str(oneProt["Protocol"]) + ","
                    try:
                        outStr += "\tProtocol: %s (%d), Data: %d bytes\n" % (list(self.ipProtDict.keys())[list(self.ipProtDict.values()).index(oneProt["Protocol"])], oneProt["Protocol"], oneProt["Data"])
                        print("\tProtocol: %s (%d), Data: %d bytes" % (list(self.ipProtDict.keys())[list(self.ipProtDict.values()).index(oneProt["Protocol"])], oneProt["Protocol"], oneProt["Data"]))
                    except ValueError:
                        outStr += "\tProtocol: %s (%d), Data: %d bytes\n" % ("Not in list", oneProt["Protocol"], oneProt["Data"])
                        print("\tProtocol: %s (%d), Data: %d bytes" % ("Not in list", oneProt["Protocol"], oneProt["Data"]))
                csvStr += " \r\n"

            if (fp != None):
                fp.write(csvStr)

            if (fpTxt != None):
                fpTxt.write(outStr)


    def setVersion(self, version):
        self.version = version

    def setInfo(self, uaInfo):
        self.uaInfo = uaInfo

    def httpUpdate(self, httpFieldName, httpFieldValue):
        if httpFieldName == "Host":
            if (self.httpHost == "?"):
                self.httpHost = httpFieldValue
            else:
                httpSplitArr = self.httpHost.split("\n")
                if httpFieldValue not in httpSplitArr:
                    self.httpHost += "\n"+httpFieldValue

        elif httpFieldName == "Origin":
            if (self.httpOrigin == "?"):
                self.httpOrigin = httpFieldValue
            else:
                httpSplitArr = self.httpOrigin.split("\n")
                if httpFieldValue not in httpSplitArr:
                    self.httpOrigin += "\n"+httpFieldValue

        elif httpFieldName == "Referer":
            if (self.httpReferer == "?"):
                self.httpReferer = httpFieldValue
            else:
                httpSplitArr = self.httpReferer.split("\n")
                if httpFieldValue not in httpSplitArr:
                    self.httpReferer += "\n"+httpFieldValue

        elif httpFieldName == "Via":
            if (self.httpVia == "?"):
                self.httpVia = httpFieldValue
            else:
                httpSplitArr = self.httpVia.split("\n")
                if httpFieldValue not in httpSplitArr:
                    self.httpVia += "\n"+httpFieldValue

        elif httpFieldName == "Location":
            if (self.httpLocation == "?"):
                self.httpLocation = httpFieldValue
            else:
                httpSplitArr = self.httpLocation.split("\n")
                if httpFieldValue not in httpSplitArr:
                    self.httpLocation += "\n"+httpFieldValue

        elif httpFieldName == "Server":
            if (self.httpServer == "?"):
                self.httpServer = httpFieldValue
            else:
                httpSplitArr = self.httpServer.split("\n")
                if httpFieldValue not in httpSplitArr:
                    self.httpServer += "\n"+httpFieldValue

    def update(self, strIp, strOs, strVersion, strUaInfo, strdevType, strHostname, strLanguage, cnt):
        if (self.ip == "?" or self.ip == "0.0.0.0" or self.ip=="127.0.0.1"):
            self.ip = strIp

        if (self.os == "?"):
            self.os = strOs

        if (self.version == "?"):
            self.version = strVersion

        if (self.uaInfo == "?" or self.uaInfo == "(unknown version)"):
            self.uaInfo = strUaInfo
        elif (self.uaInfo != "?" and strUaInfo != "?"):
            uaArr = self.uaInfo.split("\n")
            diff = True
            for ua in uaArr:
                if (ua == strUaInfo):
                    diff = False
            if (diff):
                self.uaInfo += "\n"+strUaInfo

        if (self.type == "?"):
            self.type = strdevType

        if (self.language == "?"):
            self.language = strLanguage
        elif (self.language != "?" and strLanguage != "?"):
            langArr = self.language.split("\n")
            diff = True
            for lang in langArr:
                if (lang == strLanguage):
                    diff = False
            if (diff):
                self.language += "\n"+strLanguage

        if (self.hostname == "?"):
            self.hostname = strHostname

    def addSrc(self, src, tLen, strTranProt, portNum):
        containsIP = False
        portNum = int(portNum)
        intData = int(tLen)
        for ipAddrInfo in self.stats["IpSrc"]:
            if src == ipAddrInfo["Address"]:
                containsIP = True
                # update the data
                foundPort = False
                ipAddrInfo["Volume"] += intData
                for stat in ipAddrInfo["Statistics"]:
                    if stat["Protocol"]==portNum:
                        foundPort = True
                        stat["Data"] += intData
                if not foundPort:
                    ipAddrInfo["Statistics"].append({"Protocol": portNum, "Data": intData})
        if not containsIP:
            addr = {"Address":src, "Statistics": [{"Protocol": portNum, "Data": intData}], "Volume": intData}
            self.stats["IpSrc"].append(addr)



    def addDst(self, dst, tLen, strTranProt, portNum):
        containsIP = False
        portNum = int(portNum)
        intData = int(tLen)
        for ipAddrInfo in self.stats["IpDst"]:
            if dst == ipAddrInfo["Address"]:
                containsIP = True
                # update the data
                foundPort = False
                ipAddrInfo["Volume"] += intData
                for stat in ipAddrInfo["Statistics"]:
                    if stat["Protocol"]==portNum:
                        foundPort = True
                        stat["Data"] += intData
                if not foundPort:
                    ipAddrInfo["Statistics"].append({"Protocol": portNum, "Data": intData})
        if not containsIP:
            addr = {"Address":dst, "Statistics": [{"Protocol": portNum, "Data": intData}], "Volume": intData}
            self.stats["IpDst"].append(addr)





"""
ADD NEW FILTERS HERE
"""
class ouiFilter:
    filters = []

    def __init__(self, recs):
        self.recs = recs
        self.fillFilter()

    def fillFilter(self):
        for rec in self.recs:
            f = {"oui": rec["oui"], "os": rec["os"], "version": rec["version"], "type": rec["type"]}
            self.filters.append(f)

    def parseFilter(self):
        return self.filters

class dnsFilter:
    filters = []

    def __init__(self, recs):
        self.recs = recs
        self.fillFilter()

    def fillFilter(self):
        for rec in self.recs:
            f = {"qry": rec["qry"], "os": rec["os"], "version": rec["version"]}
            self.filters.append(f)

    def parseFilter(self):
        return self.filters

class uaFilter:
    filters = []

    def __init__(self, recs):
        self.recs = recs

    def parseStrings(self):
        return self.recs[0]["str"]

class dhcpFilter:
    filters = []

    def __init__(self, recs):
        self.recs = recs

    def parseStrings(self):
        return self.recs[0]["str"]
